(function (factory) {
    if (typeof module === "object" && typeof module.exports === "object") {
        var v = factory(require, exports);
        if (v !== undefined) module.exports = v;
    }
    else if (typeof define === "function" && define.amd) {
        define(["require", "exports", "./NumberUtils.js", "./ColorUtils.js"], factory);
    }
})(function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    exports.HslColorManager = void 0;
    const NumberUtils_js_1 = require("./NumberUtils.js");
    const ColorUtils_js_1 = require("./ColorUtils.js");
    class HslColorManager {
        constructor() {
            this.key = "hsl";
            this.stringPrefix = "hsl";
        }
        handleColor(color) {
            const colorValue = color.value, hslColor = colorValue.hsl ?? color.value;
            if (hslColor.h !== undefined && hslColor.s !== undefined && hslColor.l !== undefined) {
                return (0, ColorUtils_js_1.hslToRgb)(hslColor);
            }
        }
        handleRangeColor(color) {
            const colorValue = color.value, hslColor = colorValue.hsl ?? color.value;
            if (hslColor.h !== undefined && hslColor.l !== undefined) {
                return (0, ColorUtils_js_1.hslToRgb)({
                    h: (0, NumberUtils_js_1.getRangeValue)(hslColor.h),
                    l: (0, NumberUtils_js_1.getRangeValue)(hslColor.l),
                    s: (0, NumberUtils_js_1.getRangeValue)(hslColor.s),
                });
            }
        }
        parseString(input) {
            if (!input.startsWith("hsl")) {
                return;
            }
            const regex = /hsla?\(\s*(\d+)\s*,\s*(\d+)%\s*,\s*(\d+)%\s*(,\s*([\d.%]+)\s*)?\)/i, result = regex.exec(input), minLength = 4, defaultAlpha = 1, radix = 10;
            return result
                ? (0, ColorUtils_js_1.hslaToRgba)({
                    a: result.length > minLength ? (0, NumberUtils_js_1.parseAlpha)(result[5]) : defaultAlpha,
                    h: parseInt(result[1], radix),
                    l: parseInt(result[3], radix),
                    s: parseInt(result[2], radix),
                })
                : undefined;
        }
    }
    exports.HslColorManager = HslColorManager;
});
